@@ -1,14 +1,4 @@
-from mpmath.matrices import matrix, eye, swap_row, diag, mnorm_1, mnorm_oo, \
-    mnorm_F, norm_p, zeros, ones, hilbert, randmatrix, extend
-from mpmath.linalg import inverse
-from mpmath import inf, mpf, sqrt, mpi, nstr
-
-with_numpy = False
-try:
-    import numpy
-    with_numpy = True
-except ImportError:
-    pass
+from mpmath import *
 
 def test_matrix_basic():
     A1 = matrix(3)
@@ -88,6 +78,12 @@ def test_matrix_transform():
     assert l == [2, 1]
     assert extend(eye(3), [1,2,3]) == matrix([[1,0,0,1],[0,1,0,2],[0,0,1,3]])
 
+def test_matrix_conjugate():
+    A = matrix([[1 + j, 0], [2, j]])
+    assert A.conjugate() == matrix([[mpc(1, -1), 0], [2, mpc(0, -1)]])
+    assert A.transpose_conj() == A.H == matrix([[mpc(1, -1), 2],
+                                                [0, mpc(0, -1)]])
+
 def test_matrix_creation():
     assert diag([1, 2, 3]) == matrix([[1, 0, 0], [0, 2, 0], [0, 0, 3]])
     A1 = ones(2, 3)
@@ -107,15 +103,16 @@ def test_matrix_creation():
 def test_norms():
     # matrix norms
     A = matrix([[1, -2], [-3, -1], [2, 1]])
-    assert mnorm_1(A) == 6
-    assert mnorm_oo(A) == 4
-    assert mnorm_F(A) == sqrt(20)
+    assert mnorm(A,1) == 6
+    assert mnorm(A,inf) == 4
+    assert mnorm(A,'F') == sqrt(20)
     # vector norms
+    assert norm(-3) == 3
     x = [1, -2, 7, -12]
-    assert norm_p(x, 1) == 22
-    assert round(norm_p(x, 2), 10) == 14.0712472795
-    assert round(norm_p(x, 10), 10) == 12.0054633727
-    assert norm_p(x, inf) == 12
+    assert norm(x, 1) == 22
+    assert round(norm(x, 2), 10) == 14.0712472795
+    assert round(norm(x, 10), 10) == 12.0054633727
+    assert norm(x, inf) == 12
 
 def test_vector():
     x = matrix([0, 1, 2, 3, 4])
@@ -136,9 +133,12 @@ def test_matrix_copy():
     B[0,0] = 0
     assert A != B
 
-if with_numpy:
-    def test_matrix_numpy():
-        l = [[1, 2], [3, 4], [5, 6]]
-        a = numpy.matrix(l)
-        assert matrix(l) == matrix(a)
+def test_matrix_numpy():
+    try:
+        import numpy
+    except ImportError:
+        return
+    l = [[1, 2], [3, 4], [5, 6]]
+    a = numpy.matrix(l)
+    assert matrix(l) == matrix(a)
 
